////////////////////////////////////////////////////////////////////////////////////////
// Module Name: _FW_Monitor.ino
// Project: PowerMonitor #42
// Module: Main
// Version: See VERSIONSTRING
// by Jan Breemer  
// jan@breem.nl, janbee@hack42.nl 
//
// Functions in this module:
//
// EvaluateLeds ()
// EvaluateJumpers ()
// Sequence_MQTT_Leds ()
// Sequence_Request1_Leds ()
// Sequence_Request2_Leds ()
// SystemReset ()
// setup ()
// loop ()
////////////////////////////////////////////////////////////////////////////////////////

#include "DiagMEGA.h"
#include "Client.h"
#include "Globals.h"
#include <math.h> 

// Enter the build date and time for the installed unit below.
// You will find it after compilation and upload, on the serial monitor (ctrl-shift-M)
// This item is not used anywhere in the code.
// It serves only to relate this code instance to the actually installed unit.
#define VERSIONSTRING Sep 30 2021 12:58:02

// VesionString is automatically composed in setup()
// using the date-time and locale of the build computer.
char VersionString[55];  

tPhase Ch08Mode, Ch09Mode, Ch10Mode, Ch11Mode;
bool ServiceMode, DiagnoseMode;
byte Status;

//********************************************************************************************************
void EvaluateLeds (void) // Called from /Currents.cpp/ISR(TIMER3_COMPA_vect)
{
#ifdef USE_REQUEST_BOX1
  if ((RequestLeds & 0x01) == 0x01) RED_LED_1_ON else RED_LED_1_OFF
  if ((RequestLeds & 0x04) == 0x04) GREEN_LED_1_ON else GREEN_LED_1_OFF
  if ((RequestLeds & 0x02) == 0x02) if (Blink) GREEN_LED_1_ON else GREEN_LED_1_OFF
  if ((RequestLeds & 0x08) == 0x08) 
  {
    RequestButton1Pressed = false;
    RequestLeds &= ~0x08;  // only once
  }
#endif  
#ifdef USE_REQUEST_BOX2
  if ((RequestLeds & 0x10) == 0x10) RED_LED_2_ON else RED_LED_2_OFF
  if ((RequestLeds & 0x40) == 0x40) GREEN_LED_2_ON else GREEN_LED_2_OFF
  if ((RequestLeds & 0x20) == 0x20) if (Blink) GREEN_LED_2_ON else GREEN_LED_2_OFF  
  if ((RequestLeds & 0x80) == 0x80) 
  {
    RequestButton2Pressed = false; 
    RequestLeds &= ~0x80;  // only once
  }
#endif  
  if (ConnectionError) if (Blink) CONNLED_ON else CONNLED_OFF 
  if (SubscriptionError) if (Blink) SUBLED_ON else SUBLED_OFF 
}

//********************************************************************************************************
void EvaluateJumpers (bool Show) // Called during Setup or 
{
  static bool PrevServiceMode, PrevDiagnoseMode;

  if (Show) Serial.println ("Evaluate Jumper settings: ");
  ServiceMode  = JUMPER_JP12_PRESENT; // true when JP1-JP2 jumpered
  DiagnoseMode = JUMPER_JP34_PRESENT; // true when JP3-JP4 jumpered
  if (ServiceMode != PrevServiceMode)
  {
    PrevServiceMode = ServiceMode;
    Serial.print ("Service Mode changed to ");
    if (ServiceMode) Serial.println ("ON"); else Serial.println ("OFF");
  }
  if (DiagnoseMode != PrevDiagnoseMode)
  {
    PrevDiagnoseMode = DiagnoseMode;
    Serial.print ("Diagnose Mode changed to ");
    if (DiagnoseMode) Serial.println ("ON"); else Serial.println ("OFF");
  }
  if (Show)
  {
    Serial.print("Service Mode is ");
    if (ServiceMode) Serial.println("ON"); else Serial.println("OFF");
    Serial.print("Diagnostic Mode is ");
    if (DiagnoseMode) Serial.println("ON"); else Serial.println("OFF");
    Serial.println();
  } 

#ifdef USE_CURRENTS
  // Jumperblock for phase of channels 08 .. 11
  Ch08Mode = Ch09Mode = Ch10Mode = Ch11Mode = S;
  PORTG &= ~0x01; // J1 .. J4 Low
  if ((PINL & 0x01) == 0x01) Ch08Mode = R; // No jumper
  if ((PINL & 0x02) == 0x02) Ch09Mode = R; // No jumper
  if ((PINL & 0x04) == 0x04) Ch10Mode = R; // No jumper
  if ((PINL & 0x08) == 0x08) Ch11Mode = R; // No jumper
  PORTG |= 0x01; // J1 .. J4 High
  if ((PINL & 0x01) == 0) Ch08Mode = T; // Jumper to GND
  if ((PINL & 0x02) == 0) Ch09Mode = T; // Jumper to GND
  if ((PINL & 0x04) == 0) Ch10Mode = T; // Jumper to GND
  if ((PINL & 0x08) == 0) Ch11Mode = T; // Jumper to GND
  
  if (Show)
  { 
    Serial.print ("Channel 08 Mode is: ");
    if (Ch08Mode == R) Serial.println ('R');
    if (Ch08Mode == S) Serial.println ('S');
    if (Ch08Mode == T) Serial.println ('T');
    Serial.print ("Channel 09 Mode is: ");
    if (Ch09Mode == R) Serial.println ('R');
    if (Ch09Mode == S) Serial.println ('S');
    if (Ch09Mode == T) Serial.println ('T');
    Serial.print ("Channel 10 Mode is: ");
    if (Ch10Mode == R) Serial.println ('R');
    if (Ch10Mode == S) Serial.println ('S');
    if (Ch10Mode == T) Serial.println ('T');
    Serial.print ("Channel 11 Mode is: ");
    if (Ch11Mode == R) Serial.println ('R');
    if (Ch11Mode == S) Serial.println ('S');
    if (Ch11Mode == T) Serial.println ('T');
    Serial.println();
  }  
#endif
}

//********************************************************************************************************
void Sequence_MQTT_Leds (void)
{
  Serial.println ("Testing MQTT Leds");
  for (byte i=0; i<5; i++)
  {
    CONNLED_ON
    delay(100);
    CONNLED_OFF
    delay(100);
    PUBLED_ON
    delay(100);
    PUBLED_OFF
    delay(100);
    SUBLED_ON
    delay(100);
    SUBLED_OFF
    delay(100);
    HEARTBEATLED_ON
    delay(100);
    HEARTBEATLED_OFF 
    delay(100);
  }
}

//********************************************************************************************************
void Sequence_Request1_Leds (void)
{
  Serial.println ("Testing RequestBox 1 Leds");
  for (byte i=0; i<5; i++)
  {
    RED_LED_1_ON
    delay(100);
    RED_LED_1_OFF
    delay(100);
    GREEN_LED_1_ON
    delay(100);
    GREEN_LED_1_OFF
    delay(100);
  }
}

//********************************************************************************************************
void Sequence_Request2_Leds (void)
{
  Serial.println ("Testing RequestBox 2 Leds");
  for (byte i=0; i<5; i++)
  {
    RED_LED_2_ON
    delay(100);
    RED_LED_2_OFF
    delay(100);
    GREEN_LED_2_ON
    delay(100);
    GREEN_LED_2_OFF
    delay(100);
  }
}

//********************************************************************************************************
void SystemReset (void)
{
  // Resetting the Arduino can be done by jumping to the reset vector, but
  // then the hardware will not be reset.
  // Here we pull the RESET line LOW causing a complete reset, including hardware reset.
  // This requires a connection from pin 8 to the RESET pin.
  DDRH |= 0x20;   // Data direction = Output
  PORTH &= ~0x20; // PORTH bit 5 pin 8 low
}

//********************************************************************************************************
void setup (void)
{
  Serial.begin(19200); // Diagnostics on Arduino Monitor or other TTY
  for (byte i=0; i<30; i++) Serial.println(); // k.o. clear screen
  const char CompileDateTime[] = "   Build: "__DATE__ " " __TIME__ ;
  sprintf (VersionString, "%s", "FW_");
  strcat (VersionString, CLIENTID);
  strcat (VersionString, CompileDateTime);
  Serial.println(VersionString);
  Serial.print("Client ID = ");
  Serial.println (CLIENTID);
   
  // I/O's
  DDRA = 0x0F;   // Led's in request boxes
  PORTA |= 0xC0; // Pullups for JP1, JP3 
  DDRC = 0xFF;   // ULN drive and HTBT-led 
  DDRD = 0x80;   // SUB-led
  DDRG = 0x07;   // Config-block and CONN-, PUB- leds
  PORTL |= 0x0F; // Pullups for Jumper Block Config 

  // Diagnostic outputs on ADCL (PORTF)
  USEDIAGPIN_0
  USEDIAGPIN_1
  USEDIAGPIN_2
  USEDIAGPIN_3
  USEDIAGPIN_4
  USEDIAGPIN_5
  USEDIAGPIN_6
  USEDIAGPIN_7

  InitClient();
  EvaluateJumpers(true);
  InitCurrents();
  delay(3000);  
  InitSensors();
}

//********************************************************************************************************
void loop (void)
{
  DIAGPIN_0_H 
  UpdateClient();
  UpdateCurrents();
  UpdateSensors();
  DIAGPIN_0_L
  char SerIn[10];
  if (Serial.available())
  {
    Serial.readBytes(SerIn, 10);
    if (SerIn[0] == 'R') SystemReset();
    if (SerIn[0] == '0') PORTC = 0;
    if (SerIn[0] == '1') PORTC |= 0x02;
    if (SerIn[0] == '2') PORTC |= 0x04;
    if (SerIn[0] == '3') PORTC |= 0x08;
    if (SerIn[0] == '4') PORTC |= 0x10;
    if (SerIn[0] == '5') PORTC |= 0x20;
    if (SerIn[0] == '6') PORTC |= 0x40;
    if (SerIn[0] == '7') PORTC |= 0x80;
      
    if (SerIn[0] == 'D')
    {
    #ifdef USE_DALLAS_SENSORS
      Serial.println(); 
      byte i;
      for (i=0; i < MAX_DALLAS_SENSORS; i++)
      DallasTemperatures[i] = -999;
      SearchDallasSensors();
      for (i=0; i < NumberOfDallasSensors; i++)
      ReadDallasSensor(i);
      Serial.println(); 
    #else
      Serial.println("Dallas Temperature Sensors are not used.");  
      Serial.println(); 
    #endif
    }  
  }  
  delay(1); // loop() runs at ca. 1 kHz
}
