/////////////////////////////////////////////////////////////////////////////////////////
// Module Name: Globals.h
// Project: PowerMonitor #42
// Global definitions / declarations
// by Jan Breemer 
// jan@breem.nl  janbee@hack42.nl
/////////////////////////////////////////////////////////////////////////////////////////

#ifndef GLOBALS_H
#define GLOBALS_H

#include <Arduino.h>
#include "DiagMEGA.h"

// Be sure that ClientID is reflected in the Project Name.
#define CLIENTID "Monitor_Primary"
void Sequence_MQTT_Leds ();
void Sequence_Request1_Leds ();
void Sequence_Request2_Leds ();

// Comment out the sensors which are not to be used.
// If USE_CURRENTS is not defined the Topic_Power is not published
#define USE_CURRENTS 

// The Topic_AirQuality is only published when at least one of the following 3 is defined:
#define USE_BME280  
#define USE_CCS811 
#define USE_SDS011 

// If USE_DALLAS_SENSORS is not defined the Topic_DallasTemperatures is not published
#define USE_DALLAS_SENSORS 

// If RequestBox 1 is used its Leds are tested during startup 
#define USE_REQUEST_BOX1

// If RequestBox 2 is used its Leds are tested during startup 
#define USE_REQUEST_BOX2

// General:
extern byte Status, RequestLeds;
extern char VersionString[];
extern bool ServiceMode, DiagnoseMode, Blink;
extern bool RequestButton1Pressed, RequestButton2Pressed;
void SystemReset (void);
void EvaluateJumpers (bool Show);
void EvaluateLeds(void);

// For MQTT Client / Publish:
extern char Topic_Status[];
extern char Topic_Power[];
extern char Topic_AirQuality[];
extern char Topic_DallasTemperatures[];
extern char Topic_HeartBeatAck[];
extern bool DoPublishStatus, DoPublishPower, DoPublishAirQuality,
  DoPublishDallasTemperatures, DoPublishHeartBeatAck;

// For MQTT Client / Subscribes:
extern char Topic_SetRelays[];
extern char Topic_SetRequestLights[];
extern char Topic_HeartBeat[];
extern char Payload[];

void UpdateClient (void);
void InitClient (void);
extern bool ConnectionError, SubscriptionError;

// For Current Measurements:
const float pi = 3.14159;
enum tPhase {R, S, T};
extern tPhase Ch08Mode, Ch09Mode, Ch10Mode, Ch11Mode;
extern bool DoReadaSensor, FoundZeroCrossing;
void UpdateCurrents (void);
void InitCurrents (void);

// For BME280 Temperature / Hygro / Baro Sensor:
extern bool BME280_IsInitialized;
extern int Temperature, Hygro, Baro;

// For CSS-811 eCO2 / eVOC Sensor:
extern bool CSS811_IsInitialized;
extern int eCO2, eVOC;

// For SDS011 Particle Sensor:
extern int P2p5, P10;
extern byte ParticleBuffer[10];
#define SDS_POWER_ON  PORTE &= ~0x10;
#define SDS_POWER_OFF PORTE |=  0x10;

// For Dallas Temperature Sensors:
#define MAX_DALLAS_SENSORS 10
extern int DallasTemperatures[MAX_DALLAS_SENSORS];
extern byte NumberOfDallasSensors;
void SearchDallasSensors (void);
void ReadDallasSensor (byte SensorNumber);

// For sensors general:
extern bool InitialisationError;
void UpdateSensors (void);
void InitSensors (void);

// For MQTT Leds
#define CONNLED_ON PORTG |= 0x02;
#define CONNLED_OFF PORTG &= ~0x02;
#define PUBLED_ON PORTG |= 0x04;
#define PUBLED_OFF PORTG &= ~0x04;
#define PUBLED_TOGGLE PING = 0x04;
#define SUBLED_ON PORTD |= 0x80;
#define SUBLED_OFF PORTD &= ~0x80;
#define HEARTBEATLED_ON PORTC |= 0x01;
#define HEARTBEATLED_OFF PORTC &= ~0x01;
#define HEARTBEATLED_TOGGLE PINC = 0x01;

// For Request Boxes
#define RED_LED_1_ON PORTA |= 0x04;
#define RED_LED_1_OFF PORTA &= ~0x04;
#define GREEN_LED_1_ON PORTA |= 0x01;
#define GREEN_LED_1_OFF PORTA &= ~0x01;
#define RED_LED_2_ON PORTA |= 0x08;
#define RED_LED_2_OFF PORTA &= ~0x08;
#define GREEN_LED_2_ON PORTA |= 0x02;
#define GREEN_LED_2_OFF PORTA &= ~0x02;
#define REQUEST_1_PRESSED ((PINA & 0x10) == 0x00)
#define REQUEST_2_PRESSED ((PINA & 0x20) == 0x00)

// For Jumpers
#define JUMPER_JP12_PRESENT ((PINA & 0x40) == 0x00)
#define JUMPER_JP34_PRESENT ((PINA & 0x80) == 0x00)

#endif GLOBALS_H
